import javax.swing.JPanel;
import java.awt.event.*;
import java.awt.event.MouseAdapter;
import java.awt.*;

public class MouseListener extends MouseAdapter {
   private MyWorld world;
   private Drawable currentElement;
   private JPanel panel;
   public MouseListener (MyWorld w, JPanel p){
      world = w;
      panel = p;
   } 
   public void mouseMoved(MouseEvent e) {
      Drawable newElement = world.find(e.getX(), e.getY());
      if (newElement == currentElement) return;
      if (currentElement != null) {
         currentElement.setReleased();
         currentElement = null;
      }
      if (newElement != null) { 
         currentElement = newElement;
         currentElement.setSelected();
      }
      panel.repaint();
   }
   public void mouseDragged(MouseEvent e) {
      if (currentElement == null) return;
      currentElement.dragTo(e.getX(), e.getY());
      panel.repaint();
   }
   public void mouseReleased(MouseEvent e) {
      if (currentElement != null) 
         currentElement.setReleased();
      if (!(currentElement instanceof GSpring)) {
         currentElement = null;
         return;
      }
          // we dragged an attachable element
          // we dragged a spring, so we look for and attachable element near by  
      AttachableElement element = world.findAttachableElement(e.getX(), e.getY());
      if (element == null) {
         currentElement = null;
         return;
      }
      // we dragged a spring and it is near an attachable element,
      // so we hook them to the right spring end.
      GSpring spring = (GSpring) currentElement;
      Vector2D a=spring.getAendPosition();
      if ((a.getX()==e.getX()) && (a.getY()==e.getY()))
         spring.attachAend(element);
      Vector2D b=spring.getBendPosition();
      if ((b.getX()==e.getX()) && (b.getY()==e.getY()))
         spring.attachBend(element);
      currentElement = null;
      panel.repaint();      
   }
}